@@ -1,12 +1,12 @@
 from contextlib import contextmanager
 from ctypes import CFUNCTYPE, POINTER, Structure, pointer, cast, memmove, memset, sizeof, addressof, cdll, byref, string_at, c_char_p, c_int, c_double, c_int64, c_void_p, c_char
+from ctypes.util import find_library
 from functools import partial
 from hashlib import sha256
 import hmac
 from datetime import datetime
 import os
 from re import sub
-from sys import platform
 from time import time
 from urllib.parse import urlencode, urlsplit, quote
 from uuid import uuid4
@@ -21,7 +21,7 @@ def sqlite_s3_query(url, get_credentials=lambda now: (
     os.environ['AWS_SECRET_ACCESS_KEY'],
     os.environ.get('AWS_SESSION_TOKEN'),  # Only needed for temporary credentials
 ), get_http_client=lambda: httpx.Client(),
-   get_libsqlite3=lambda: cdll.LoadLibrary({'linux': 'libsqlite3.so.0', 'darwin': 'libsqlite3.dylib'}[platform])):
+   get_libsqlite3=lambda: cdll.LoadLibrary(find_library('sqlite3'))):
     libsqlite3 = get_libsqlite3()
     libsqlite3.sqlite3_errstr.restype = c_char_p
     libsqlite3.sqlite3_errmsg.restype = c_char_p
@@ -86,8 +86,8 @@ def sqlite_s3_query(url, get_credentials=lambda now: (
         request_headers = aws_sigv4_headers(
             now, access_key_id, secret_access_key, region, method, to_auth_headers, params,
         )
-        url = f'{scheme}://{netloc}{path}?{urlencode(params)}'
-        with http_client.stream(method, url, headers=request_headers) as response:
+        url = f'{scheme}://{netloc}{path}'
+        with http_client.stream(method, url, params=params, headers=request_headers) as response:
             response.raise_for_status()
             yield response
 
@@ -188,6 +188,14 @@ def sqlite_s3_query(url, get_credentials=lambda now: (
             p_size[0] = size
             return SQLITE_OK
 
+        x_lock_type = CFUNCTYPE(c_int, c_void_p, c_int)
+        def x_lock(p_file, e_lock):
+            return SQLITE_OK
+
+        x_unlock_type = CFUNCTYPE(c_int, c_void_p, c_int)
+        def x_unlock(p_file, e_lock):
+            return SQLITE_OK
+
         x_file_control_type = CFUNCTYPE(c_int, c_void_p, c_int, c_void_p)
         def x_file_control(p_file, op, p_arg):
             return SQLITE_NOTFOUND
@@ -196,6 +204,11 @@ def sqlite_s3_query(url, get_credentials=lambda now: (
         def x_device_characteristics(p_file):
             return 0
 
+        x_access_type = CFUNCTYPE(c_int, c_void_p, c_char_p, c_int, POINTER(c_int))
+        def x_access(p_vfs, z_name, flags, z_out):
+            z_out[0] = 0
+            return SQLITE_OK
+
         x_full_pathname_type = CFUNCTYPE(c_int, c_void_p, c_char_p, c_int, POINTER(c_char))
         def x_full_pathname(p_vfs, z_name, n_out, z_out):
             memmove(z_out, file_name.encode() + b'\0', len(file_name) + 1)
@@ -214,8 +227,8 @@ def sqlite_s3_query(url, get_credentials=lambda now: (
             ('x_truncate', c_void_p, None),
             ('x_sync', c_void_p, None),
             ('x_file_size', x_file_size_type, x_file_size_type(x_file_size)),
-            ('x_lock', c_void_p, None),
-            ('x_unlock', c_void_p, None),
+            ('x_lock', x_lock_type, x_lock_type(x_lock)),
+            ('x_unlock', x_unlock_type, x_unlock_type(x_unlock)),
             ('x_check_reserved_lock', c_void_p, None),
             ('x_file_control', x_file_control_type, x_file_control_type(x_file_control)),
             ('x_sector_size', c_void_p, None),
@@ -233,7 +246,7 @@ def sqlite_s3_query(url, get_credentials=lambda now: (
             ('p_app_data', c_char_p, None),
             ('x_open', x_open_type, x_open_type(x_open)),
             ('x_delete', c_void_p, None),
-            ('x_access', c_void_p, None),
+            ('x_access', x_access_type, x_access_type(x_access)),
             ('x_full_pathname', x_full_pathname_type, x_full_pathname_type(x_full_pathname)),
             ('x_dl_open', c_void_p, None),
             ('x_dl_error', c_void_p, None),
@@ -254,7 +267,7 @@ def sqlite_s3_query(url, get_credentials=lambda now: (
     @contextmanager
     def get_db(vfs):
         db = c_void_p()
-        run(libsqlite3.sqlite3_open_v2, f'file:/{file_name}?immutable=1'.encode() + b'\0', byref(db), SQLITE_OPEN_READONLY | SQLITE_OPEN_URI, vfs_name.encode() + b'\0')
+        run(libsqlite3.sqlite3_open_v2, f'file:/{file_name}'.encode() + b'\0', byref(db), SQLITE_OPEN_READONLY | SQLITE_OPEN_URI, vfs_name.encode() + b'\0')
         try:
             yield db
         finally:
@@ -263,7 +276,7 @@ def sqlite_s3_query(url, get_credentials=lambda now: (
     @contextmanager
     def get_pp_stmt(db, sql):
         pp_stmt = c_void_p()
-        run_with_db(db, libsqlite3.sqlite3_prepare_v3, db, sql.encode() + b'\0', -1, 0, byref(pp_stmt), None)
+        run_with_db(db, libsqlite3.sqlite3_prepare_v2, db, sql.encode() + b'\0', -1, byref(pp_stmt), None)
         try:
             yield pp_stmt
         finally:
